local AddonName, NickAlert = ...
local FriendlyAddonName = "Nick Alert"


-- Config Panel Text Strings
NickAlert.Text={
Title = "Nick Alert",
TitleSub = "Change how Nick Alert behaves by setting the options below",
Enable = "Enable the Add On to function",
AutoAdd = "Add your character names to the list automatically",
Toasts = "Alert you when people go online/offline(Must be checked for any toast alerts)",
TFriends = "All Friends online/offline toasts(checked) or just marked",
TGuild = "All Guild online/offline toasts(checked) or just marked",
TBNet = "All Real-ID/Battle-Tag online/offline toasts(checked) or just marked",
Guild = "Scan Guild channel",
Party = "Scan Party channel",
Say  = "Scan Say channel",
Yell = "Scan Yell channel",
Emotes = "Scan Emotes",
BGs = "Scan Battleground channel",
Raid = "Scan Raid channel",
Chats = "Scan Other channels",
WhisperList = "Alert you when specified people whisper you",
BNet = "Alert you when specified Real ID / Battle Tag people msg you",
ChatDebug = "Print all matches to default chat log",
DisplayMode = "Enable display of mode text in alert (ie Party, Guild, BNet)",
MF = "Enables output to the moveable message frame",
MFScale = "Percent scale of the message frame (50 - 300)",
FontSize = "Font Size in message frame",
FadeDelay = "Fade message frame text after X seconds",
Log = "Put all the alerts into the log window",
LogShow = "Show the log as soon as you log in",
MSBT  = "Enable output to MSBT's notification frame",
Sound = "Enable sound to notify you of an alert",
CustomMP3 = "Enable custom alert.mp3 sound",
CustomOGG = "Enable custom alert.ogg sound",
Throttle = "Enable alert throttle",
ThrottleTime = "Amount of time to throttle over",
ThrottleAmount = "Max number of alerts in above time"
}

-- Local text colors
local yel = "|cFFFFFF00"
local wht = "|cFFFFFFFF"
local blu = "|cFF00FFFF"
local red = "|cFFFF0000"
local grn = "|cFF00FF00"
local res = "|r"



-- Bill's Utils
local Wait = BillsUtils.Wait
local ChkBox = BillsUtils.ChkBox
local ChkBoxSave = BillsUtils.ChkBoxSave
local CheckClick = BillsUtils.CheckClick
local SliderSave = BillsUtils.SliderSave
local SliderMW = BillsUtils.SliderMW
local Slider = BillsUtils.Slider
local SliderEditBox = BillsUtils.SliderEditBox
local SJprint = BillsUtils.SJprint

BillsUtils.Locals[#BillsUtils.Locals +1] = function ()
	Wait = BillsUtils.Wait
	ChkBox = BillsUtils.ChkBox
	ChkBoxSave = BillsUtils.ChkBoxSave
	CheckClick = BillsUtils.CheckClick
	SliderSave = BillsUtils.SliderSave
	SliderMW = BillsUtils.SliderMW
	Slider = BillsUtils.Slider
	SliderEditBox = BillsUtils.SliderEditBox
	SJprint = BillsUtils.SJprint
end

-- General Settings / functions
local X = 16 -- standard Horizontal Spacing
local Y = 35 -- standard Vertical spacing 26
local TxtY = 13
local NAS -- NickAlert.Settings
local NAD -- NickAlert.Defaults
local NN -- NickAlert.NickNames
local CF -- NickAlert.ChatFilter
local WL -- NickAlert.WhisperList
local PerChar -- NickAlert.PerChar
local NAT -- NickAlert.ToastsList 

local HighestButton = 16

-- Color Changing Button stuff
local ButtonTextures = {
	RED = {
		up = "Interface\\Buttons\\UI-Panel-Button-Up",
		down = "Interface\\Buttons\\UI-Panel-Button-Down",
		highlight = "Interface\\Buttons\\UI-Panel-Button-Highlight",
	},
	GREEN = {
		up = "Interface\\AddOns\\NickAlert\\Green-Panel-Button-Up",
		down = "Interface\\AddOns\\NickAlert\\Green-Panel-Button-Down",
		highlight = "Interface\\AddOns\\NickAlert\\Green-Panel-Button-Highlight",
	},
}

local function GetSkin(self)
	return self.__skin
end

local function SetSkin(self, skin)
	if self.__skin == skin then return end
	self.__skin = skin

	if self:IsMouseOver() and IsMouseButtonDown() then
		local down = ButtonTextures[skin].down
		self.Left:SetTexture(down)
		self.Right:SetTexture(down)
		self.Middle:SetTexture(down)
	else
		local up = ButtonTextures[skin].up
		self.Left:SetTexture(up)
		self.Right:SetTexture(up)
		self.Middle:SetTexture(up)
	end

	self:SetHighlightTexture(ButtonTextures[skin].highlight, "ADD")
	self:GetHighlightTexture():SetTexCoord(0, 80/128, 0, 22/32)
end

local function OnMouseDown(self, button)
	local down = ButtonTextures[self.__skin].down
	self.Left:SetTexture(down)
	self.Right:SetTexture(down)
	self.Middle:SetTexture(down)
end

local function OnMouseUp(self, button)
	local up = ButtonTextures[self.__skin].up
	self.Left:SetTexture(up)
	self.Right:SetTexture(up)
	self.Middle:SetTexture(up)
end

local function NewButton(parent, skin, name)
	local button = CreateFrame("Button", name, parent or UIParent)
	button:SetSize(40, 22)

	local left = button:CreateTexture(nil, "BACKGROUND")
	left:SetPoint("TOPLEFT")
	left:SetPoint("BOTTOMLEFT")
	left:SetWidth(12)
	left:SetTexCoord(0, 12/128, 0, 22/32)
	button.Left = left

	local right = button:CreateTexture(nil, "BACKGROUND")
	right:SetPoint("TOPRIGHT")
	right:SetPoint("BOTTOMRIGHT")
	right:SetWidth(12)
	right:SetTexCoord(68/128, 80/128, 0, 22/32)
	button.Right = right

	local middle = button:CreateTexture(nil, "BACKGROUND")
	middle:SetPoint("TOPLEFT", left, "TOPRIGHT")
	middle:SetPoint("BOTTOMRIGHT", right, "BOTTOMLEFT")
	middle:SetTexCoord(12/128, 68/128, 0, 22/32)
	button.Middle = middle

	local text = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	text:SetAllPoints(true)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("CENTER")
	button:SetFontString(text)
	button:SetNormalFontObject("GameFontNormal")
	button:SetHighlightFontObject("GameFontHighlight")	

	button:SetScript("OnMouseDown", OnMouseDown)
	button:SetScript("OnMouseUp", OnMouseUp)
	button:SetScript("OnShow", OnMouseUp)

	button.GetSkin = GetSkin
	button.SetSkin = SetSkin

	SetSkin(button, skin or "RED")

	return button
end

NickAlert.ConfigFrame = CreateFrame("Frame")
NickAlert.ConfigFrame:RegisterEvent("ADDON_LOADED")

NickAlert.ConfigFrame:SetScript("OnEvent",function(self, event, ...)
	local arg = ...
	if event ~= "ADDON_LOADED" and arg ~= AddonName then return end
	NAS = NickAlert.Settings
	NAD = NickAlert.Defaults
	NN = NickAlert.NickNames
	CF = NickAlert.ChatFilter
	WL = NickAlert.WhisperList
	PerChar = NickAlert.PerChar
	NAT = NickAlert.ToastsList
end)

NickAlert.Checkclick = function()
	PlaySound( "GAMEGENERICBUTTONPRESS" )
end

--Panel
NickAlert.panel = CreateFrame( "Frame", "NAlertConfig", UIParent )
NickAlert.panel2 = CreateFrame( "Frame", "NAlertConfig2", NickAlert.panel )
local me = NickAlert.panel
local me2 = NickAlert.panel2
me:SetScript("OnLoad", function()
	NickAlert.init()
end)


me.name = "Nick Alert"
me2:SetSize(380, 475)
me2:SetPoint("BOTTOMRIGHT", me, "BOTTOMRIGHT")

-- Pane title
	local Title = me:CreateFontString( nil, "ARTWORK", "GameFontNormalLarge" )
	Title:SetPoint( "TOPLEFT", 16, -16 )
	Title:SetText( NickAlert.Text.Title )
	local SubText = me:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
	SubText:SetPoint( "TOPLEFT", Title, "BOTTOMLEFT", 0, -8 )
	SubText:SetPoint( "RIGHT", -32, 0 )
	SubText:SetHeight( 32 )
	SubText:SetJustifyH( "LEFT" )
	SubText:SetJustifyV( "TOP" )
	SubText:SetText( NickAlert.Text.TitleSub )

-- Scroll Frame
	me.ScrollFrame = CreateFrame("ScrollFrame", "NAlertConfigScrollFrame", me, "UIPanelScrollFrameTemplate")
	me.ScrollFrame:SetScrollChild(me2)
	me.ScrollFrame:SetPoint("TOPRIGHT", me ,"TOPRIGHT", -30,- 64)
	me.ScrollFrame:SetPoint("BOTTOMRIGHT", me ,"BOTTOMRIGHT", -30, 10)
	me.ScrollFrame:SetPoint("BOTTOMLEFT", me ,"BOTTOMLEFT", 30, 10)

-- Check Boxes & Text
	local last = me2
	local Command = {"Enable", "AutoAdd", "Toasts", "TFriends", "TGuild", "TBNet", "Guild", "Party", "BGs", "Raid", "Say", "Yell", "Emotes",
	"Chats", "WhisperList", "BNet", "ChatDebug", "DisplayMode", "MF"} 

	for x = 1, #Command do
		me[ Command[x] ] = ChkBox( last , "NickAlertCfg", Command[x] , me2, CheckClick, NickAlert.Text )
		last = me[ Command[x] ]
	end
	local FontSizeDemo
	local SliderFontSize = function(self, delta)
		local min,max = self:GetMinMaxValues()
		local step = self:GetValueStep()
		local value = self:GetValue()
	
		if delta < 0 then 
			value = value - step
		elseif delta > 0 then
			value = (value + step)
		end
		if value < min then
			value = min
		elseif value > max then
			value = max
		end
		self:SetValue( value )
		FontSizeDemo:SetFont( STANDARD_TEXT_FONT, value )
		FontSizeDemo:SetText( "Font Size Demo "..tostring(value) )
	end
	
	me.MFScale = Slider( last , "NickAlertCfg", "MFScale", me2 , SliderMW , NickAlert.Text, 50, 300, 1 )
	last = me.MFScale
	me.MFScale.Val = SliderEditBox( last , "NickAlertCfg", "MFScale", me2 )
	
	me.FontSize = Slider( last , "NickAlertCfg", "FontSize", me2 , SliderFontSize , NickAlert.Text, 7, 24, 1 )
	me.FontSize:SetScript( "OnValueChanged", function( self )
		local val = self:GetValue()
		self.Val:SetNumber( val )
		FontSizeDemo:SetFont( STANDARD_TEXT_FONT, val )
		FontSizeDemo:SetText( "Font Size Demo "..tostring( val ) )
		PlaySound( "GAMEGENERICBUTTONPRESS" )
	end)
	
	last = me.FontSize
	me.FontSize.Val = SliderEditBox( last , "NickAlertCfg", "FontSizeDemo", me2 )
	
	FontSizeDemo = me2:CreateFontString( "NalertFontSizeDemo", "ARTWORK") --, "GameFontHighlightSmall" )
	FontSizeDemo:SetFont( STANDARD_TEXT_FONT, 10 )
	FontSizeDemo:SetPoint( "TOPLEFT", last, "BOTTOMLEFT", 0, -0 )
	FontSizeDemo:SetPoint( "RIGHT", -32, 0 )
	FontSizeDemo:SetHeight( 32 )
	FontSizeDemo:SetJustifyH( "LEFT" )
	FontSizeDemo:SetJustifyV( "CENTER" )
	FontSizeDemo:SetText( "Font Size Demo" )

	last = FontSizeDemo
	
	
	me.FadeDelay = Slider( last , "NickAlertCfg", "FadeDelay", me2 , SliderMW , NickAlert.Text, 1, 60, 1 )
	last = me.FadeDelay
	me.FadeDelay.Val = SliderEditBox( last , "NickAlertCfg", "FadeDelay", me2 )
	
	Command = {"Log", "LogShow", "MSBT", "Sound", "CustomMP3", "CustomOGG", "Throttle"}
	for x = 1, #Command do
		me[ Command[x] ] = ChkBox( last , "NickAlertCfg", Command[x] , me2, CheckClick, NickAlert.Text )
		last = me[ Command[x] ]
	end
	
-- Throttle Time Slider	and EditBox
	me.ThrottleTime = Slider( last , "NickAlertCfg", "ThrottleTime", me2 , SliderMW , NickAlert.Text, 1, 60, 1 )
	last = me.ThrottleTime
	me.ThrottleTime.Val = SliderEditBox( last , "NickAlertCfg", "ThrottleTime", me2 )
	
-- Throttle Amount Slider
	me.ThrottleAmount = Slider( last , "NickAlertCfg", "ThrottleAmount", me2 , SliderMW , NickAlert.Text, 1, 60, 1 )
	last = me.ThrottleAmount
	me.ThrottleAmount.Val = SliderEditBox( last , "NickAlertCfg", "ThrottleAmount", me2 )
	
--	End of frame



-- XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
-- X  Init / Reset the config panel to match current settings                                                     X
-- XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
--


NickAlert.Init = function(...)
	local Command = {"Enable", "AutoAdd", "Toasts", "TFriends", "TGuild", "TBNet", "Guild", "Party", "BGs",
				"Raid", "Say", "Yell", "Emotes", "Chats", 
				"WhisperList", "BNet", "ChatDebug", "DisplayMode", "MF",
				"MSBT", "Sound", "CustomMP3", "CustomOGG", "Throttle",
				}
				
	for x = 1, #Command do
		me[Command[x]]:SetChecked(NAS[Command[x]])
	end
	
	local Command = {"MFScale", "FontSize", "FadeDelay", "ThrottleTime", "ThrottleAmount" }
	
	for x = 1, #Command do
		me[Command[x]]:SetValue( NAS[Command[x]])
		me[Command[x]].Val:SetCursorPosition( 0 )
		me[Command[x]].Val:SetNumber( NAS[Command[x]] )
	end
end

-- Reset config panel and settings to DEFAULT
NickAlert.Reset = function(...)
	local cmd = { "Enable", "AutoAdd", "Toasts", "TFriends", "TGuild", "TBNet", "Guild", "Party", "BGs", "Raid", "Say",
		"Yell", "Emotes", "Chats", "WhisperList", "BNet", "ChatDebug", "DisplayMode", 
		"MF", "MFScale", "FontSize", "FadeDelay", "MSBT", "Sound", "CustomMP3", "CustomOGG", 
		"Throttle", "ThrottleTime", "ThrottleAmount" }
	for x = 1, #cmd do
		NAS[cmd[x]] = NAD[cmd[x]]
	end	
	NickAlert.Init()
end



me.refresh = function(self)
	NickAlert.Init()
end

me.default = function (self)
	NickAlert.Reset()
end

me.cancel = function (self)
	NickAlert.Init()
end


-- Save settings from config panel
me.okay = function(self)
	local cmd = { "Enable", "AutoAdd", "Toasts", "TFriends", "TGuild", "TBNet", "Guild", "Party", "Say", "Yell", "Emotes", "BGs", "Chats", "WhisperList", "BNet",
		 "ChatDebug", "DisplayMode", "MF", "MSBT", "Sound", "CustomMP3", "CustomOGG", "Throttle"}
	for x = 1, #cmd do
		ChkBoxSave( me, cmd[x] , NAS)
	end
	
	SliderSave( me, "MFScale", NAS) -- Message Frame Scale Slider
	SliderSave( me, "FontSize", NAS) -- Font Size Slider
	SliderSave( me, "FadeDelay", NAS) -- Throttle Time Slider
	SliderSave( me, "ThrottleTime", NAS) -- Throttle Time Slider
	SliderSave( me, "ThrottleAmount", NAS) -- Throttle Amount Slider
	
	NickAlert.SettingsUpdate()
	NickAlert.EventManager()
end

-- Register the config panel
InterfaceOptions_AddCategory( me )

-- ****************************************************************************
-- * List Manager Child Panel                                                 *
-- ****************************************************************************

local GSelected = GetGuildInfo("player")
local SliderFrame1
local SliderFrame2

StaticPopupDialogs["NICKALERT_DELETEGUILD"] = {
	text = "Do you want to delete the guild entry %s ?",
	button1 = "Yes",
	button2 = "No",
	OnAccept = function()
		NAT[GSelected] = nil
		GSelected = GetGuildInfo("player")
		NickAlert.ListDisplayUpdate()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}

StaticPopupDialogs["NICKALERT_CANNOTDELETEGUILD"] = {
	text = "You cannot delete the guild you are currently in.",
	button1 = "OK",
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/ 
}

--Panel
NickAlert.panel3 = CreateFrame( "Frame", "NAlertConfig3", UIParent )
local me = NickAlert.panel3

me.name = "List Manager"
me.parent = NickAlert.panel.name

me.ListButtons = {}

-- Pane title
local Title = me:CreateFontString( nil, "ARTWORK", "GameFontNormalLarge" )
Title:SetPoint( "TOPLEFT", 16, -16 )
Title:SetText( "Nick Names" )
local SubText = me:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
SubText:SetPoint( "TOPLEFT", Title, "BOTTOMLEFT", 0, -8 )
SubText:SetPoint( "RIGHT", -32, 0 )
SubText:SetHeight( 32 )
SubText:SetJustifyH( "LEFT" )
SubText:SetJustifyV( "TOP" )
SubText:SetText( "Manage your alert lists here"  )
	
local last = me
local width = _G["InterfaceOptionsFramePanelContainer"]:GetWidth() / 3 

me.SelectButton = {}
local Selected = 1
local SelectButtonWidth = _G["InterfaceOptionsFramePanelContainer"]:GetWidth() / 5
local SelectedMultiplier = 1.5 -- this is how much larger the selected mode is

local SelectButtonText = { "Nicknames", "Chan Filters", "Whispers", "Battle Net", "Friend Toasts", "Guild Toasts", "Battle Net Toasts" }
me.SelectButtonUpdate = function()
	for x = 1, 7 do
		if Selected == x then
			me.SelectButton[x]:SetSize( SelectButtonWidth * SelectedMultiplier , 20)
			me.SelectButton[x]:SetButtonState("PUSHED")
			me.SelectButton[x]:SetSkin("GREEN")
		else
			me.SelectButton[x]:SetSize( SelectButtonWidth , 20)
			me.SelectButton[x]:SetButtonState("NORMAL")
			me.SelectButton[x]:SetSkin("RED")
		end
	end
	me.slider:SetValue(0)
	me.slider2:SetValue(0)
	NickAlert.ListDisplayUpdate()
end

for x = 1, 7 do
	me.SelectButton[x] = NewButton( me, "RED")
	if x == 1 then
		me.SelectButton[x]:SetPoint("TOPLEFT", last, "TOPLEFT", 20, -60 )
	elseif x == 5 then
		me.SelectButton[x]:SetPoint("TOPLEFT", me.SelectButton[1], "BOTTOMLEFT" )
	else
		me.SelectButton[x]:SetPoint("TOPLEFT", me.SelectButton[x-1], "TOPRIGHT" )
	end
	if Selected == x then
		me.SelectButton[x]:SetSize( SelectButtonWidth * SelectedMultiplier , 20)
		me.SelectButton[x]:SetButtonState("PUSHED")
		me.SelectButton[x]:SetSkin("GREEN")
	else
		me.SelectButton[x]:SetSize( SelectButtonWidth , 20)
		me.SelectButton[x]:SetButtonState("NORMAL")
		me.SelectButton[x]:SetSkin("RED")
	end
	me.SelectButton[x]:SetText( SelectButtonText[x] )
	me.SelectButton[x].count = x
	me.SelectButton[x]:SetScript("OnClick", function(self, button, down)
		if (button ~= "LeftButton") then
			return
		end
		PlaySound( "GAMEGENERICBUTTONPRESS" )
		if Selected ~= self.count and Selected <= 3 then
			me.EBox:SetText("")
		end
		Selected = self.count
		me.SelectButtonUpdate()
		return
	end)
	
end
last = me.SelectButton[5]	

local ButtonSubText = me:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
ButtonSubText:SetPoint( "TOPLEFT", me.SelectButton[5], "BOTTOMLEFT", 0, -8 )
ButtonSubText:SetPoint( "RIGHT", -32, 0 )
ButtonSubText:SetHeight( 32 )
ButtonSubText:SetJustifyH( "LEFT" )
ButtonSubText:SetJustifyV( "TOP" )
ButtonSubText:SetText( "Manage your alert lists here"  )

NickAlert.BNetToggle = function( ID )
	local _, _, _, _, _, _, _, _, _, _, _, _, noteText, _, _ = BNGetFriendInfoByID( ID )
	if Selected == 4 then -- BNet Msg
		key = "NAlert"
	else -- BNet Toast
		key = "NAToast"
	end
	
	local marked
	
	if not( noteText ) then
		marked = false
	else
		marked = string.find( noteText, key ) 
	end
	
	if marked then -- remove the mark
		noteText = string.gsub(noteText, key, " " )
		while string.find(noteText, "^%s") do
			noteText = string.sub(noteText, 2)
		end	
		while string.find(noteText, "%s$") do
			noteText = string.sub(noteText, 1, -2)
		end
		while string.find(noteText, "%s%s" ) do
			noteText = string.gsub(noteText, "%s%s", " ")
		end
	else
		if noteText == nil or noteText == "" then
			noteText = key
		else
			noteText = noteText.." "..key
		end
	end
	BNSetFriendNote( ID, noteText)
	return
end

NickAlert.ListToggle = function( list, value )
	local Table
	if list == "FToasts" then
		Table = PerChar.Toasts
	elseif list == "GToasts" then
		Table = NAT[GSelected]
	else
		return
	end
	
	if Table[value] then
		if list == "FToasts" then
			Table[value] = nil
		else
			Table[value] = false
		end
	else
		Table[value] = true
	end
	return
end

NickAlert.ListDisplayUpdate = function()
	local Table
	if Selected == 1 then
		Table = NN
		ButtonSubText:SetText( "Add / Remove nicknames/triggers to scan for here"  )
		me.EBox.Text:SetText( "Enter a nickname or click a button above" )
	elseif Selected == 2 then
		Table = CF
		ButtonSubText:SetText( "These are the /# channels to watch \nChannels you are watching / Channels you are currently in"  )
		me.EBox.Text:SetText( "Enter a channel name or click a button above" )
	elseif Selected == 3 then
		Table = WL
		ButtonSubText:SetText( "The addon will alert you if any of these people whisper you"  )
		me.EBox.Text:SetText( "Enter a nickname or click a button above" )
	-- Selected == 4 is included with Selected == 7
	elseif Selected == 5 then
		local FriendCount = GetNumFriends()
		if not(PerChar.Toasts) then
			PerChar.Toasts = {}
		end
		Table = {}
		for x = 1, FriendCount do
			Table[x] = (GetFriendInfo( x ))
		end
		table.sort(Table)
		ButtonSubText:SetText( "A list of friends that you can get toasts for \nClick the person's name to toggle. GREEN marked will alert you"  )
	elseif Selected == 6 then
		Table = {}
		for index, _ in pairs(NAT) do
			if index then
				Table[#Table +1] = index
			end
		end
		table.sort(Table)
		ButtonSubText:SetText( "These are the guilds you are/were in on this server / people in that guild"  )
	elseif Selected == 4 or Selected == 7 then
		Table = {}
		local FriendCount = BNGetNumFriends()
		for x = 1, FriendCount do
			Table[x] = {}
			Table[x].presenceID, Table[x].fullName, _, _, _, _, _, _, _, _, _, _, Table[x].noteText, _, _ = BNGetFriendInfo( x )
		end
		if Selected == 4 then
			ButtonSubText:SetText( "A list of Real IDs and Battle Tags than can alert you if they message you\nClick the person's name to toggle. GREEN will alert you"  )
		else
			ButtonSubText:SetText( "A list of Real IDs and Battle Tags than can alert you if they log on/off \nClick the person's name to toggle. Green marked will alert you"  )
		end
	end
		
	local maxcount = #Table
	local slider = me.slider:GetValue()
		
	for x = 1 , HighestButton do
		if x + slider > maxcount then
			me.ListButtons[x]:SetText( " " )
			me.ListButtons[x].val = ""
			me.ListButtons[x]:SetSkin("RED")
		else
			if Selected >= 1 and Selected <= 3 then
				me.ListButtons[x]:SetSkin("RED")
				me.ListButtons[x]:SetText( Table[x + slider] )
				me.ListButtons[x].val = Table[x + slider]
			-- Selected == 4 is with Selected == 7
			elseif Selected == 5 then
				if PerChar.Toasts[Table[x + slider]] then
					me.ListButtons[x]:SetSkin( "GREEN" )
				else
					me.ListButtons[x]:SetSkin( "RED" )
				end
				me.ListButtons[x]:SetText( Table[x + slider] )
				me.ListButtons[x].val = Table[x + slider]
			elseif Selected == 6 then
				if not( GSelected ) then
					GSelected = (GetGuildInfo("player"))
				end
				if Table[x + slider] == GSelected then
					me.ListButtons[x]:SetSkin( "GREEN" )
				else
					me.ListButtons[x]:SetSkin( "RED" )
				end
				me.ListButtons[x]:SetText( Table[x + slider] )
				me.ListButtons[x].val = Table[x + slider]
			elseif Selected == 4 or Selected == 7 then
				local AlertText = "NAlert"
				if Selected == 7 then
					AlertText = "NAToast"
				end
					
				if Table[x + slider].noteText and string.find( Table[x + slider].noteText, AlertText) then
					me.ListButtons[x]:SetSkin( "GREEN" )
				else
					me.ListButtons[x]:SetSkin( "RED" )
				end
				me.ListButtons[x]:SetText( Table[x + slider].fullName )
				me.ListButtons[x].val = Table[x + slider].presenceID
			end
		end
	end
	me.slider:SetMinMaxValues( 0, (#Table > HighestButton and #Table - HighestButton or 1 ))
	if #Table <= HighestButton then
		me.slider:SetValue(0)
		me.slider:Disable(true)
	else
		me.slider:Enable(true)
	end
	
	-- ***************************************************************************
	-- * Chan List Buttons
	if Selected == 2 then
		SliderFrame2:EnableMouseWheel(1)
		local chanlist = { GetChannelList() }
		local chanmax = #chanlist / 2
		for x = 1, HighestButton do
			me.ListButtons2[x]:SetSkin("RED")
			if x > chanmax then
				me.ListButtons2[x]:SetText(" ")
				me.ListButtons2[x].val = ""
			else
				me.ListButtons2[x]:SetText(chanlist[x * 2])
				me.ListButtons2[x].val = chanlist[x * 2]
			end
			me.ListButtons2[x]:Show()
		end
	elseif Selected == 6 then
		SliderFrame2:EnableMouseWheel(1)
		if not(GSelected) then
			GSelected = (GetGuildInfo("player"))
		end
		if not(NAT[GSelected]) then
			NAT[GSelected] = {}
		end
		
		local GNum
		local GTable = {}
		local S2Val = me.slider2:GetValue()
		if GSelected == GetGuildInfo("player") then
			GNum = GetNumGuildMembers()
			for x = 1, GNum do
				GTable[x] = (GetGuildRosterInfo(x))
			end
		else
			for index, _ in pairs(NAT[GSelected]) do
				GTable[#GTable +1] = index
			end
			GNum = #GTable
		end
		table.sort(GTable)
		
		for x = 1, HighestButton do
			if x + S2Val > GNum then
				me.ListButtons2[x]:SetSkin("RED")
				me.ListButtons2[x]:SetText(" ")
				me.ListButtons2[x].val = ""
			else
				if NAT[GSelected][GTable[x + S2Val]] then
					me.ListButtons2[x]:SetSkin("GREEN")
				else
					me.ListButtons2[x]:SetSkin("RED")
				end
				me.ListButtons2[x]:SetText(GTable[x + S2Val])
				me.ListButtons2[x].val = (GTable[x + S2Val])
			end
			me.ListButtons2[x]:Show()
		end
	else
		SliderFrame2:EnableMouseWheel(nil)
		for x = 1, HighestButton do
			me.ListButtons2[x]:Hide()
		end
	end
	
	-- ***************************************************************************
	-- * Add / Delete Ebox buttons
	if Selected >= 1 and Selected <= 3 then
		me.AddButton:Show()
		me.AddButton:SetText("Add")
		me.DeleteButton:Show()
		me.EBox:Show()
		me.EBox.Text:Show()
	elseif Selected == 6 then
		me.AddButton:Show()
		me.AddButton:SetText("Delete Guild")
		me.DeleteButton:Hide()
		me.EBox:Hide()
		me.EBox.Text:Hide()
	else
		me.AddButton:Hide()
		me.DeleteButton:Hide()
		me.EBox:Hide()
		me.EBox.Text:Hide()
	end
	if Selected ~= 6 then
		me.slider2:Hide()
	else
		me.slider2:Show()
		local count = 0
		if GSelected == (GetGuildInfo("player")) then
			count = GetNumGuildMembers()
		else
			for _,_ in pairs(NAT[GSelected]) do
				count = count + 1
			end
		end
		me.slider2:SetMinMaxValues( 0, (count > HighestButton and count - HighestButton or 1) )
		if count < 11 then
			me.slider2:SetValue(0)
			me.slider2:Disable(true)
		else
			me.slider2:Enable(true)
		end
	end	
end


for x = 1, HighestButton do
	me.ListButtons[x] = NewButton( me, "RED")
	if x == 1 then
		me.ListButtons[x]:SetPoint("TOPLEFT", last, "TOPLEFT", 60, -60 )
	else
		me.ListButtons[x]:SetPoint("TOPLEFT", last, "BOTTOMLEFT" )
	end
	me.ListButtons[x]:SetSize( width , 20)
	me.ListButtons[x]:SetText( " " )
	me.ListButtons[x].val = ""
	me.ListButtons[x]:SetScript("OnClick", function(self, button, down)
		if (button ~= "LeftButton") then
			return
		end
		if self.val == "" then return end
		PlaySound( "GAMEGENERICBUTTONPRESS" )
		if Selected > 3 then
			if Selected == 4 then
				NickAlert.BNetToggle( self.val )
			elseif Selected == 5 then
				NickAlert.ListToggle("FToasts", self.val)
			elseif Selected == 6 then
				GSelected = self.val
			elseif Selected == 7 then
				NickAlert.BNetToggle( self.val )
			end
			NickAlert.ListDisplayUpdate()
			return
		end
		
		me.EBox:SetText(self.val)
		return
	end)
	last = me.ListButtons[x]
end
me.slider = CreateFrame( "Slider", "NAlertNNListSlider", me, "OptionsSliderTemplate" )
me.slider:SetMinMaxValues( 0, 1 )
me.slider:SetOrientation("VERTICAL")
me.slider:SetPoint("TOPLEFT", me.ListButtons[1] ,"TOPRIGHT", 5 , 0)
me.slider:SetPoint("BOTTOMLEFT", me.ListButtons[HighestButton] ,"BOTTOMRIGHT", 5 , 0)
me.slider:SetValueStep( 1 )
me.slider:SetWidth( 16 )	
me.slider:SetScript( "OnValueChanged", function( self )
	NickAlert.ListDisplayUpdate()
end)
getglobal(me.slider:GetName() .. 'Low'):SetText( " " )
getglobal(me.slider:GetName() .. 'High'):SetText( " " )
me.slider:SetScript("OnMouseWheel", function(self, delta)
	local number = 0
	if Selected == 1 then
		number = #NN
	elseif Selected == 2 then
		number = #CF
	elseif Selected == 3 then
		number = #WL
	elseif Selected == 4 then
		number = BNGetNumFriends()
	elseif Selected == 5 then
		number = GetNumFriends()
	elseif Selected == 6 then
		for _,_ in pairs(NAT) do
			number = number + 1
		end
	elseif Selected == 7 then
		number = BNGetNumFriends()
	end
	if number <= HighestButton then
		return
	end
	SliderMW(self, -delta)
end)
SliderFrame1 = CreateFrame( "Frame", "NAlertSliderFrame1", me )
SliderFrame1:SetPoint("TOPLEFT", me.ListButtons[1], "TOPLEFT", -10, 10)
SliderFrame1:SetPoint("BOTTOMRIGHT", me.slider, "BOTTOMRIGHT", 10, -10)
SliderFrame1:EnableMouseWheel(1)
SliderFrame1:SetScript("OnMouseWheel", function(self, delta)
	local number = 0
	if Selected == 1 then
		number = #NN
	elseif Selected == 2 then
		number = #CF
	elseif Selected == 3 then
		number = #WL
	elseif Selected == 4 then
		number = BNGetNumFriends()
	elseif Selected == 5 then
		number = GetNumFriends()
	elseif Selected == 6 then
		for _,_ in pairs(NAT) do
			number = number + 1
		end
	elseif Selected == 7 then
		number = BNGetNumFriends()
	end
	if number <= HighestButton then
		return
	end
	SliderMW(me.slider, -delta)
end)

me.ListButtons2={}
for x = 1, HighestButton do
	me.ListButtons2[x] = NewButton( me, "RED") -- CreateFrame("Button", "NAlertListButton2-"..tostring(x), me, "UIPanelButtonTemplate" )
	me.ListButtons2[x]:SetPoint("TOPLEFT",me.ListButtons[x] , "TOPRIGHT", 40, 0 )
	me.ListButtons2[x]:SetSize( width , 20)
	me.ListButtons2[x]:SetText( " " )
	me.ListButtons2[x].count = x
	me.ListButtons2[x]:SetScript("OnClick", function(self, button, down)
		if (button ~= "LeftButton") then
			return
		end
		if self.val == "" then return end
		PlaySound( "GAMEGENERICBUTTONPRESS" )
		if Selected == 6 then
			NickAlert.ListToggle("GToasts", self.val)
		else
			me.EBox:SetText(self.val)
		end
		NickAlert.ListDisplayUpdate()
		return
	end)
end

me.slider2 = CreateFrame( "Slider", "NAlertNNListSlider2", me, "OptionsSliderTemplate" )
me.slider2:SetMinMaxValues( 0, 1 )
me.slider2:SetOrientation("VERTICAL")
me.slider2:SetPoint("TOPLEFT", me.ListButtons2[1] ,"TOPRIGHT", 5 , 0)
me.slider2:SetPoint("BOTTOMLEFT", me.ListButtons2[HighestButton] ,"BOTTOMRIGHT", 5 , 0)
me.slider2:SetValueStep( 1 )
me.slider2:SetWidth( 16 )	
me.slider2:SetScript( "OnValueChanged", function( self )
	NickAlert.ListDisplayUpdate()
end)
getglobal(me.slider2:GetName() .. 'Low'):SetText( " " )
getglobal(me.slider2:GetName() .. 'High'):SetText( " " )
me.slider2:SetScript("OnMouseWheel", function(self, delta)
	local number = 0
	if GSelected ==  (GetGuildInfo("player")) then
		number = GetNumGuildMembers()
	else
		for _,_ in pairs(NAT[GSelected]) do
			number = number + 1
		end
	end
	if number <= HighestButton then
		return
	end
	SliderMW(self, -delta)
end)
SliderFrame2 = CreateFrame( "Frame", "NAlertSliderFrame2", me )
SliderFrame2:SetPoint("TOPLEFT", me.ListButtons2[1], "TOPLEFT", -10, 10)
SliderFrame2:SetPoint("BOTTOMRIGHT", me.slider2, "BOTTOMRIGHT", 10, -10)
SliderFrame2:EnableMouseWheel(1)
SliderFrame2:SetScript("OnMouseWheel", function(self, delta)
	local number = 0
	if GSelected ==  (GetGuildInfo("player")) then
		number = GetNumGuildMembers()
	else
		for _,_ in pairs(NAT[GSelected]) do
			number = number + 1
		end
	end
	if number <= HighestButton then
		return
	end
	SliderMW(me.slider2, -delta)
end)

me.EBox = CreateFrame( "EditBox", "NAlertNNEditbox", me, "InputBoxTemplate" )
me.EBox:SetFontObject("ChatFontNormal")
me.EBox:SetTextInsets(0, 0, 3, 3)
me.EBox:SetPoint("BOTTOMLEFT", me , "BOTTOMLEFT", 30 , 50 )
me.EBox:SetPoint("RIGHT", -32, 0)
me.EBox:SetHeight(19)
me.EBox:SetWidth(50)
me.EBox:SetAutoFocus(false)
me.EBox:SetMaxLetters( 254 )
me.EBox:SetScript("OnEnterPressed" , function(self)
	self:ClearFocus()
end)
me.EBox:SetScript("OnEscapePressed", function(self)
	self:SetText( "" )
	self:ClearFocus()
end)

me.EBox.Text = me:CreateFontString( nil, "ARTWORK", "GameFontHighlightSmall" )
me.EBox.Text:SetPoint( "BOTTOMLEFT", me.EBox , "BOTTOMLEFT", 0 , Y/4 )
me.EBox.Text:SetPoint( "RIGHT", -32, 0 )
me.EBox.Text:SetHeight( 32 )
me.EBox.Text:SetJustifyH( "LEFT" )
me.EBox.Text:SetJustifyV( "TOP" )
me.EBox.Text:SetText( "Enter data or click a button above" )

me.AddButton = CreateFrame("Button", "NalertAddButton", me, "UIPanelButtonTemplate" )
me.AddButton:SetPoint("BOTTOMLEFT", me, "BOTTOMLEFT", 20 , 20 )
me.AddButton:SetSize( 150, 25)
me.AddButton:SetText("Add")
me.AddButton:SetScript("OnClick", function(self, button, down)
	if (button ~= "LeftButton") then
		return
	end
	if Selected == 6 then --Guild List
		if GSelected == GetGuildInfo("player") then
			StaticPopup_Show("NICKALERT_CANNOTDELETEGUILD")
		else
			StaticPopup_Show("NICKALERT_DELETEGUILD", GSelected)
		end
		NickAlert.ListDisplayUpdate()
		return
	end
	local EBoxText = me.EBox:GetText()
	if EBoxText == "" or EBoxText == " " then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	local mode = { "NickList", "Filter", "WhisperList"}
	NickAlert.List(mode[Selected], "add", EBoxText )
	me.EBox:SetText("")
	me.EBox:ClearFocus()
	NickAlert.ListDisplayUpdate()
	return
end)

me.DeleteButton = CreateFrame("Button", "NAlertDeleteButton", me, "UIPanelButtonTemplate" )
me.DeleteButton:SetPoint("BOTTOMRIGHT", me, "BOTTOMRIGHT", -20 , 20 )
me.DeleteButton:SetSize( 150, 25)
me.DeleteButton:SetText("Delete")
me.DeleteButton:SetScript("OnClick", function(self, button, down)
	if (button ~= "LeftButton") then
		return
	end
	local EBoxText = me.EBox:GetText()
	if EBoxText == "" or EBoxText == " " then
		return
	end
	PlaySound( "GAMEGENERICBUTTONPRESS" )
	local mode = { "NickList", "Filter", "WhisperList"}
	NickAlert.List(mode[Selected], "rem", EBoxText )
	me.EBox:SetText("")
	me.EBox:ClearFocus()
	NickAlert.ListDisplayUpdate()
	return
end)

me.refresh= function( self )
	me.slider:SetValue(0)
	me.slider2:SetValue(0)
	NickAlert.ListDisplayUpdate()
	me.init()
end


me.init = function(self)
	GSelected = GetGuildInfo("player")
	if not(PerChar) then
		PerChar = NickAlert_PerChar
	end
	if not(NAT) then
		NAT = NickAlert_Toasts
	end
	if not(NAT[GSelected]) then
		NAT[GSelected] = {}
	end
	local Gnum = GetNumGuildMembers()
	local friend
	local tmptable = {}
	for x = 1, Gnum do
		friend = (GetGuildRosterInfo(x))
		if not(NAT[GSelected][friend] ) then
			NAT[GSelected][friend] = false
		end
		tmptable[#tmptable +1] = friend
	end
	for index, _ in pairs( NAT[GSelected] ) do
		if not( tContains( tmptable, index )) then
			NAT[GSelected][index] = nil
		end
	end
	me.SelectButtonUpdate()
end


me.refresh = me.init

InterfaceOptions_AddCategory( me )

